#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>

#include "config.h"
#include "client_daemon.h"
#include "daemon.h"

void usage(void);
void openlogfile(void);
void checkdaemon(char *);
int initsocket(void);
int initmodule(char *);

int main(int argc, char **argv)
{
  pid_t pid;
  int s;

  if (argc > 1)
    usage();

  /* Redirect stderr to a log file */
#ifdef DRCOM_DEBUG
  fprintf(stderr, "%10u DEBUG drcomcd: Redirecting stderr to %s...\n", (unsigned) time(NULL), DRCOMCD_LOG_FILE);
#endif
  openlogfile();

#ifdef DRCOM_DEBUG
  fprintf(stderr, "%10u DEBUG drcomcd: Checking for running instance of drcomcd...\n", (unsigned) time(NULL));
#endif
  checkdaemon(argv[0]);

  /* Try to open a socket for listening */
#ifdef DRCOM_DEBUG
  fprintf(stderr, "%10u DEBUG drcomcd: Opening %s for listening...\n", (unsigned) time(NULL), DRCOMCD_SOCK);
#endif
  s = initsocket();

  /* Try to load drcom.o */
#ifdef DRCOM_DEBUG
  fprintf(stderr, "%10u DEBUG drcomcd: Loading %s module...\n", (unsigned) time(NULL), DRCOM_MODULE);
#endif
  initmodule(DRCOM_MODULE);

  /* Fork a child */
#ifdef DRCOM_DEBUG
  fprintf(stderr, "%10u DEBUG drcomcd: Forking...\n", (unsigned) time(NULL));
#endif
  pid = fork();
  if (pid > 0)
    return 0;
  else if (pid < 0)
  {
    fprintf(stderr, "drcomcd: Error forking the daemon\n");
    exit(EXIT_FAILURE);
  }

  drcomcd_daemon(s);

  return 0;
}

void usage()
{
  puts("drcomcd, daemon part of the drcomc-drcomcd client-daemon programs\n\n"
       "  usage: drcomcd\n\n"
       "  No option should be given.\n");

  exit(EXIT_FAILURE);

  return;
}

void openlogfile()
{
  int savefd;

  savefd = dup(fileno(stderr));
  if (savefd < 0)
  {
    perror("drcomcd: copying stderr file descriptor");
    fprintf(stderr, "Cannot copy stderr fd, logging to stderr instead\n");
    return;
  }

  /* More thought should be put into error handling here */
  stderr = freopen(DRCOMCD_LOG_FILE, "w", stderr);
  if (!stderr)
  {
    stderr = fdopen(savefd, "w");
    if (!stderr)
    {
      printf("drcomcd: reopening stderr: %s\n", strerror(errno));
      exit(EXIT_FAILURE);
    }
    fprintf(stderr, "Cannot open log file, logging to stderr instead\n");
  }

  /* I never knew there was such a function */
  setvbuf(stderr, NULL, _IONBF, 0);

  return;
}

void checkdaemon(char *name)
{
  char s[50];
  int r;
  pid_t pid = getpid();

  sprintf(s, "ps -e | grep `basename %s` | grep -v ^\\ *%d\\ ", name, pid);
  fprintf(stderr, "%s\n", s);
  r = system(s);
  if (r == 0)
  {
    fprintf(stderr, "drcomcd: daemon is already running\n");
    exit(EXIT_FAILURE);
  }

  return;
}

int initsocket()
{
  int s, r;
  struct sockaddr_un un_daemon;

  un_daemon.sun_family = AF_UNIX;
  strncpy(un_daemon.sun_path, DRCOMCD_SOCK, sizeof(un_daemon.sun_path));

  r = remove(DRCOMCD_SOCK);
  if (r && errno != ENOENT)
  {
    perror("drcomcd: Removing " DRCOMCD_SOCK);
    exit(EXIT_FAILURE);
  }

  s = socket(PF_UNIX, SOCK_STREAM, 0);
  if (s == -1)
  {
    perror("drcomcd: Socket creation");
    exit(EXIT_FAILURE);
  }
  r = bind(s, (struct sockaddr *) &un_daemon, sizeof(un_daemon));
  if (r)
  {
    perror("drcomcd: Bind");
    exit(EXIT_FAILURE);
  }
  r = listen(s, 1);
  if (r)
  {
    perror("drcomcd: Listen");
    exit(EXIT_FAILURE);
  }

  return s;
}

int initmodule(char *modulename)
{
  int r;
  char s[50];

  /* We should try to be as generic as possible */
  sprintf(s, "modprobe %s", modulename);
  r = system(s);
  if (r)
  {
    fprintf(stderr, "drcomcd: Error loading drcom module\n");
    exit(EXIT_FAILURE);
  }

  return 0;
}

