#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include <pthread.h>

#include <drcom.h>

#include "config.h"
#include "drcom_types.h"

struct drcom_handle *h;

/* Data for drcom.o */

struct drcom_status_data status_data[1];
struct drcom_auth_data auth_data[1];
struct drcom_iface_data iface_data[1];
struct drcom_except_data except_data[MAX_EXCEPT_ITEMS];

void msgprint(const char *);
void client_watchport(void);
void * client_keepalive(void *);
void client_logout(int);

static pid_t clientpid;

int main(void)
{
  FILE *f;
  pthread_t th_keepalive;
  struct drcom_session_info *s;
  int r, r2, authlen;

/*
  printf("Loading drcom.o...\n");
  r = system("insmod drcom");
  if (r)
  {
    printf("Error loading drcom.o!\n");
    printf("Maybe drcom.o is already loaded? Trying to unload it first...\n");
    r2 = system("rmmod drcom");
    if (r2)
    {
      printf("What the... drcomcd aborting!\n");
      return r;
    }
    printf("Trying to load drcom.o again...\n");
    r2 = system("insmod drcom");
    if (r2)
    {
      printf("Loading drcom.o failed, AGAIN.\nAborting.\n");
      return r2;
    }
  }
*/

  h = drcom_create_handle();

  r = drcom_init(h, &msgprint);
  if (r)
    return r;

  status_data->status = STATUS_NOTLOGIN;

  f = fopen("/proc/drcom/status", "w");
  fwrite(status_data, STATUS_LEN, 1, f);
  fclose(f);

  r = drcom_login(h, -1);
  if (r)
    return r;

  s = drcom_get_session_info(h);
  authlen = drcom_get_authlen();

  /* struct drcom_auth is already packed, so no worries here */
  memcpy(auth_data->auth, s->auth, authlen);

  /* memset(drcom_iface_data, 0, 4 * sizeof(struct addrport)); */

  iface_data->hostip = s->hostip;
  iface_data->hostport = s->hostport;
  iface_data->servip = s->servip;
  iface_data->servport = s->servport;
  iface_data->dnsp = s->dnsp;
  iface_data->dnss = s->dnss;

  /* memset(drcom_except_data, 0, 20 * sizeof(struct addrmask)); */

  except_data[0].addr = 0;
  except_data[0].mask = 0;

  /* Actually write to the files */

  f = fopen("/proc/drcom/auth", "w");
  fwrite(auth_data, AUTH_LEN, 1, f);
  fclose(f);

  f = fopen("/proc/drcom/iface", "w");
  fwrite(iface_data, IFACE_LEN, 1, f);
  fclose(f);

  f = fopen("/proc/drcom/except", "w");
  fwrite(except_data, EXCEPT_LEN, MAX_EXCEPT_ITEMS, f);
  fclose(f);

  /* Now tell drcom.o to start authenticating */

  status_data->status = STATUS_LOGIN;

  f = fopen("/proc/drcom/status", "w");
  fwrite("1", STATUS_LEN, 1, f);
  fclose(f);

  /* Fork a thread */

  clientpid = getpid();
  signal(SIGINT, client_logout);

  pthread_create(&th_keepalive, NULL, client_keepalive, NULL);
  client_watchport();

  return 0;
}

void msgprint(const char *s)
{
  puts(s);
  return;
}

void client_watchport()
{
  drcom_watchport(h);
  return;
}

void * client_keepalive(void *arg)
{
  sleep(9);
  drcom_keepalive(h);
  return NULL;
  arg = NULL;
}

void client_logout(int signum)
{
  FILE *f;
  int r;

  if (getpid() != clientpid)
    pthread_exit(0);

  status_data->status = STATUS_NOTLOGIN;

  f = fopen("/proc/drcom/status", "w");
  fwrite(status_data, STATUS_LEN, 1, f);
  fclose(f);

  drcom_logout(h, -1);
  drcom_cleanup(h);

  drcom_destroy_handle(h);

  printf("Unloading drcom.o...\n");
  r = system("rmmod drcom");
  if (r)
  {
    printf("Error unloading drcom.o!\n");
    exit(r);
  }

  exit(0);
  return;
  signum = 0;
}

