#!/bin/sh

if [ $# -ne 2 ]
then
  echo "error: makesubdir.sh takes 2 arguments"
  exit 1
fi

KERNELRELEASE=$1
AVAIL_SUBDIRS=$2

# These variables are _not_ to be used to determine the kernel
# directory, since REVISION is really just the numeric revision;
# the extra version is scrapped.
MAJOR=`echo $KERNELRELEASE | cut -f 1 -d .`
MINOR=`echo $KERNELRELEASE | cut -f 2 -d .`
REVISION=`echo $KERNELRELEASE | cut -f 3 -d . | sed -e 's/[^0-9].*//'`

FILES="drcom_linux.h proc.h proto.h init.c proc.c proto.c except/except_macros.h except/common.c except/private.c"
SUBDIR=
CURDIR=`pwd`

case $MAJOR in
  2) case $MINOR in
       4) SUBDIR="2.4.x" ;;
       6) case $REVISION in
            0 | 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8) SUBDIR="2.6.x_early" ;;
            *) SUBDIR="2.6.x" ;;
          esac ;;
       *) echo "error: kernel is not a 2.4.x or 2.6.x kernel"; exit 1 ;;
     esac ;;
  *) echo "error: kernel is not a 2.x.y kernel"; exit 1 ;;
esac

# Errors shouldn't be redirected to /dev/null
echo $AVAIL_SUBDIRS | grep -w "$SUBDIR" > /dev/null
if [ $? -ne 0 ]
then
  echo "error: Subdir $SUBDIR is invalid"
  exit 1
fi

mkdir -p $SUBDIR
for i in $FILES
do
  DIRNAME=`dirname $i`
  if [ "$DIRNAME" != "." ]
  then
    mkdir -p $SUBDIR/$DIRNAME
  fi
  ln -sf $CURDIR/$i $SUBDIR/$i
done

ln -sf $CURDIR/makefiles/Makefile.$SUBDIR $SUBDIR/Makefile

echo $SUBDIR

