#include "drcom_linux.h"

/* for procfs functions and variables */
#include <linux/proc_fs.h>
/* for copy_to_user & copy_from_user */
#include <asm/uaccess.h>

#include "config.h"
#include "drcom_types.h"
#include "proc.h"
#include "proto.h"

int read_all(char *, int, int *, void *, int);
int write_all(const char *, unsigned long, void *, int);
int status_read(char *, char **, off_t, int, int *, void *);
int status_write(struct file *, const char *, unsigned long, void *);
int auth_read(char *, char **, off_t, int, int *, void *);
int auth_write(struct file *, const char *, unsigned long, void *);
int iface_read(char *, char **, off_t, int, int *, void *);
int iface_write(struct file *, const char *, unsigned long, void *);
int except_read(char *, char **, off_t, int, int *, void *);
int except_write(struct file *, const char *, unsigned long, void *);
int init_proc(void);
int cleanup_proc(void);

#ifndef MAX
#define MAX(a, b) (((a) > (b)) ? a : b)
#endif

#ifndef MIN
#define MIN(a, b) (((a) < (b)) ? a : b)
#endif

struct proc_dir_entry *proc_drcom,
                      *proc_drcom_status,
                      *proc_drcom_auth,
                      *proc_drcom_iface,
                      *proc_drcom_except;

struct drcom_status_data _status_data[1], *status_data = _status_data;
struct drcom_auth_data _auth_data[1], *auth_data = _auth_data;
struct drcom_iface_data _iface_data[1], *iface_data = _iface_data;
struct drcom_except_data _except_data[MAX_EXCEPT_ITEMS + 1], *except_data = _except_data; /* +1 for tail */

int read_all(char *page, int count, int *eof, void *data, int maxlen)
{
  int len = MIN(count, maxlen);

  int i;
  for (i = 0; i < len; ++i)
    page[i] = *((char *) data + i);

  *eof = 1;

  return len;
}

int write_all(const char *buffer, unsigned long count, void *data, int maxlen)
{
  int len = MIN(count, (unsigned long) maxlen);

  int i;
  for (i = 0; i < len; ++i)
    *((char *) data + i) = buffer[i];

  return len;
}

int status_read(char *page, char **start, off_t off, int count, int *eof, void *data)
{
  return read_all(page, count, eof, data, STATUS_LEN);
  /* Turn off the warnings and still get the same object file */
  off = 0; start = 0;
}

int status_write(struct file *file, const char *buffer, unsigned long count, void *data)
{
  return write_all(buffer, count, data, STATUS_LEN);
  /* Turn off the warnings and still get the same object file */
  file = 0;
}

int auth_read(char *page, char **start, off_t off, int count, int *eof, void *data)
{
  return read_all(page, count, eof, data, AUTH_LEN);
  /* Turn off the warnings and still get the same object file */
  off = 0; start = 0;
}

int auth_write(struct file *file, const char *buffer, unsigned long count, void *data)
{
  return write_all(buffer, count, data, AUTH_LEN);
  /* Turn off the warnings and still get the same object file */
  file = 0;
}

int iface_read(char *page, char **start, off_t off, int count, int *eof, void *data)
{
  return read_all(page, count, eof, data, IFACE_LEN);
  /* Turn off the warnings and still get the same object file */
  off = 0; start = 0;
}

int iface_write(struct file *file, const char *buffer, unsigned long count, void *data)
{
  return write_all(buffer, count, data, IFACE_LEN);
  /* Turn off the warnings and still get the same object file */
  file = 0;
}

int except_read(char *page, char **start, off_t off, int count, int *eof, void *data)
{
  return read_all(page, count, eof, data, MAX_EXCEPT_LEN);
  /* Turn off the warnings and still get the same object file */
  off = 0; start = 0;
}

int except_write(struct file *file, const char *buffer, unsigned long count, void *data)
{
  return write_all(buffer, count, data, MAX_EXCEPT_LEN);
  /* Turn off the warnings and still get the same object file */
  file = 0;
}

int init_proc(void)
{
  proc_drcom = proc_mkdir("drcom", &proc_root);

  proc_drcom_status = create_proc_entry("status", 0, proc_drcom);
  proc_drcom_auth = create_proc_entry("auth", 0, proc_drcom);
  proc_drcom_iface = create_proc_entry("iface", 0, proc_drcom);
  proc_drcom_except = create_proc_entry("except", 0, proc_drcom);

  proc_drcom_status->read_proc = &status_read;
  proc_drcom_status->write_proc = &status_write;
  proc_drcom_status->data = status_data;
  proc_drcom_auth->read_proc = &auth_read;
  proc_drcom_auth->write_proc = &auth_write;
  proc_drcom_auth->data = auth_data;
  proc_drcom_iface->read_proc = &iface_read;
  proc_drcom_iface->write_proc = &iface_write;
  proc_drcom_iface->data = iface_data;
  proc_drcom_except->read_proc = &except_read;
  proc_drcom_except->write_proc = &except_write;
  proc_drcom_except->data = except_data;

  /* no need to initialize others */

  status_data->status = STATUS_NOTLOGIN;

  except_data[MAX_EXCEPT_ITEMS].addr = 0;
  except_data[MAX_EXCEPT_ITEMS].mask = 0;

  return 0;
}

int cleanup_proc(void)
{
  remove_proc_entry("except", proc_drcom);
  remove_proc_entry("iface", proc_drcom);
  remove_proc_entry("auth", proc_drcom);
  remove_proc_entry("status", proc_drcom);
  remove_proc_entry("drcom", &proc_root);

  return 0;
}

int get_proc_bufs(struct drcom_status_data **proto_status,
                  struct drcom_auth_data **proto_auth,
                  struct drcom_iface_data **proto_iface,
                  struct drcom_except_data **proto_except)
{
  *proto_status = status_data;
  *proto_auth = auth_data;
  *proto_iface = iface_data;
  *proto_except = except_data;

  return 0;
}

